package sun.net.httpserver;

import java.util.TimerTask;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.Headers;
import javax.net.ssl.SSLEngine;
import java.net.URISyntaxException;
import com.sun.net.httpserver.Filter;
import java.net.URI;
import java.io.BufferedInputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.nio.channels.CancelledKeyException;
import java.nio.channels.SocketChannel;
import java.util.logging.Level;
import java.security.AccessController;
import java.security.PrivilegedAction;
import com.sun.net.httpserver.HttpContext;
import com.sun.net.httpserver.HttpHandler;
import java.util.Iterator;
import java.net.BindException;
import java.io.IOException;
import java.util.LinkedList;
import java.util.Collections;
import java.util.HashSet;
import java.util.logging.Logger;
import java.util.Timer;
import com.sun.net.httpserver.HttpServer;
import java.util.List;
import java.util.Set;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.net.InetSocketAddress;
import javax.net.ssl.SSLContext;
import com.sun.net.httpserver.HttpsConfigurator;
import java.util.concurrent.Executor;

class ServerImpl implements TimeSource
{
    private String protocol;
    private boolean https;
    private Executor executor;
    private HttpsConfigurator httpsConfig;
    private SSLContext sslContext;
    private ContextList contexts;
    private InetSocketAddress address;
    private ServerSocketChannel schan;
    private Selector selector;
    private SelectionKey listenerKey;
    private Set<HttpConnection> idleConnections;
    private Set<HttpConnection> allConnections;
    private Set<HttpConnection> reqConnections;
    private Set<HttpConnection> rspConnections;
    private List<Event> events;
    private Object lolock;
    private volatile boolean finished;
    private volatile boolean terminating;
    private boolean bound;
    private boolean started;
    private volatile long time;
    private volatile long subticks;
    private volatile long ticks;
    private HttpServer wrapper;
    static final int CLOCK_TICK;
    static final long IDLE_INTERVAL;
    static final int MAX_IDLE_CONNECTIONS;
    static final long TIMER_MILLIS;
    static final long MAX_REQ_TIME;
    static final long MAX_RSP_TIME;
    static final boolean timer1Enabled;
    private Timer timer;
    private Timer timer1;
    private Logger logger;
    ServerImpl.Dispatcher dispatcher;
    static boolean debug;
    private int exchangeCount;
    
    ServerImpl(HttpServer wrapper, String s, InetSocketAddress obj, int backlog) throws IOException {
        this.lolock = new Object();
        this.finished = false;
        this.terminating = false;
        this.bound = false;
        this.started = false;
        this.subticks = 0L;
        this.exchangeCount = 0;
        this.protocol = s;
        this.wrapper = wrapper;
        ServerConfig.checkLegacyProperties(this.logger = Logger.getLogger("com.sun.net.httpserver"));
        this.https = s.equalsIgnoreCase("https");
        this.address = obj;
        this.contexts = new ContextList();
        this.schan = ServerSocketChannel.open();
        if (obj != null) {
            this.schan.socket().bind(obj, backlog);
            this.bound = true;
        }
        this.selector = Selector.open();
        this.schan.configureBlocking(false);
        this.listenerKey = this.schan.register(this.selector, 16);
        this.dispatcher = new ServerImpl.Dispatcher();
        this.idleConnections = Collections.synchronizedSet(new HashSet());
        this.allConnections = Collections.synchronizedSet(new HashSet());
        this.reqConnections = Collections.synchronizedSet(new HashSet());
        this.rspConnections = Collections.synchronizedSet(new HashSet());
        this.time = System.currentTimeMillis();
        (this.timer = new Timer("server-timer", true)).schedule(new ServerImpl.ServerTimerTask(), (long)ServerImpl.CLOCK_TICK, (long)ServerImpl.CLOCK_TICK);
        if (ServerImpl.timer1Enabled) {
            (this.timer1 = new Timer("server-timer1", true)).schedule(new ServerImpl.ServerTimerTask1(), ServerImpl.TIMER_MILLIS, ServerImpl.TIMER_MILLIS);
            this.logger.config("HttpServer timer1 enabled period in ms:  " + ServerImpl.TIMER_MILLIS);
            this.logger.config("MAX_REQ_TIME:  " + ServerImpl.MAX_REQ_TIME);
            this.logger.config("MAX_RSP_TIME:  " + ServerImpl.MAX_RSP_TIME);
        }
        this.events = new LinkedList();
        this.logger.config("HttpServer created " + s + " " + obj);
    }
    
    public void bind(InetSocketAddress endpoint, int backlog) throws IOException {
        if (this.bound) {
            throw new BindException("HttpServer already bound");
        }
        if (endpoint == null) {
            throw new NullPointerException("null address");
        }
        this.schan.socket().bind(endpoint, backlog);
        this.bound = true;
        return;
    }
    
    public void start() {
        if (!this.bound || this.started || this.finished) {
            throw new IllegalStateException("server in wrong state");
        }
        if (this.executor == null) {
            this.executor = new ServerImpl.DefaultExecutor();
        }
        Thread thread = new Thread(this.dispatcher);
        this.started = true;
        thread.start();
        return;
    }
    
    public void setExecutor(Executor executor) {
        if (this.started) {
            throw new IllegalStateException("server already started");
        }
        this.executor = executor;
        return;
    }
    
    public Executor getExecutor() {
        return this.executor;
    }
    
    public void setHttpsConfigurator(HttpsConfigurator httpsConfig) {
        if (httpsConfig == null) {
            throw new NullPointerException("null HttpsConfigurator");
        }
        if (this.started) {
            throw new IllegalStateException("server already started");
        }
        this.httpsConfig = httpsConfig;
        this.sslContext = httpsConfig.getSSLContext();
        return;
    }
    
    public HttpsConfigurator getHttpsConfigurator() {
        return this.httpsConfig;
    }
    
    public void stop(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("negative delay parameter");
        }
        this.terminating = true;
        try {
            this.schan.close();
        }
        catch (IOException ex) {}
        this.selector.wakeup();
        while (System.currentTimeMillis() < System.currentTimeMillis() + (long)(n * 1000)) {
            this.delay();
            if (this.finished) {
                break;
            }
            continue;
        }
        this.finished = true;
        this.selector.wakeup();
        synchronized (this.allConnections) {
            Iterator<HttpConnection> iterator = this.allConnections.iterator();
            while (iterator.hasNext()) {
                ((HttpConnection)iterator.next()).close();
            }
        }
        this.allConnections.clear();
        this.idleConnections.clear();
        this.timer.cancel();
        if (ServerImpl.timer1Enabled) {
            this.timer1.cancel();
        }
        return;
    }
    
    public synchronized HttpContextImpl createContext(String str, HttpHandler httpHandler) {
        if (httpHandler == null || str == null) {
            throw new NullPointerException("null handler, or path parameter");
        }
        HttpContextImpl httpContextImpl = new HttpContextImpl(this.protocol, str, httpHandler, this);
        this.contexts.add(httpContextImpl);
        this.logger.config("context created: " + str);
        return httpContextImpl;
    }
    
    public synchronized HttpContextImpl createContext(String str) {
        if (str == null) {
            throw new NullPointerException("null path parameter");
        }
        HttpContextImpl httpContextImpl = new HttpContextImpl(this.protocol, str, null, this);
        this.contexts.add(httpContextImpl);
        this.logger.config("context created: " + str);
        return httpContextImpl;
    }
    
    public synchronized void removeContext(String str) throws IllegalArgumentException {
        if (str == null) {
            throw new NullPointerException("null path parameter");
        }
        this.contexts.remove(this.protocol, str);
        this.logger.config("context removed: " + str);
        return;
    }
    
    public synchronized void removeContext(HttpContext httpContext) throws IllegalArgumentException {
        if (!(httpContext instanceof HttpContextImpl)) {
            throw new IllegalArgumentException("wrong HttpContext type");
        }
        this.contexts.remove((HttpContextImpl)httpContext);
        this.logger.config("context removed: " + httpContext.getPath());
        return;
    }
    
    public InetSocketAddress getAddress() {
        return (InetSocketAddress)AccessController.doPrivileged(new PrivilegedAction<InetSocketAddress>() {
            @Override
            public InetSocketAddress run() {
                return (InetSocketAddress)ServerImpl.this.schan.socket().getLocalSocketAddress();
            }
        });
    }
    
    Selector getSelector() {
        return this.selector;
    }
    
    void addEvent(Event event) {
        synchronized (this.lolock) {
            this.events.add(event);
            this.selector.wakeup();
        }
    }
    
    static synchronized void dprint(String x) {
        if (ServerImpl.debug) {
            System.out.println(x);
        }
    }
    
    static synchronized void dprint(Exception x) {
        if (ServerImpl.debug) {
            System.out.println(x);
            x.printStackTrace();
        }
    }
    
    Logger getLogger() {
        return this.logger;
    }
    
    private void closeConnection(HttpConnection httpConnection) {
        httpConnection.close();
        this.allConnections.remove(httpConnection);
        switch (httpConnection.getState()) {
            case REQUEST: {
                this.reqConnections.remove(httpConnection);
                break;
            }
            case RESPONSE: {
                this.rspConnections.remove(httpConnection);
                break;
            }
            case IDLE: {
                this.idleConnections.remove(httpConnection);
                break;
            }
        }
        assert !this.reqConnections.remove(httpConnection);
        assert !this.rspConnections.remove(httpConnection);
        assert !this.idleConnections.remove(httpConnection);
        return;
    }
    
    void logReply(int i, String s, String str) {
        if (!this.logger.isLoggable(Level.FINE)) {
            return;
        }
        if (str == null) {
            str = "";
        }
        String string;
        if (s.length() > 80) {
            string = s.substring(0, 80) + "<TRUNCATED>";
        }
        else {
            string = s;
        }
        this.logger.fine(string + " [" + i + " " + Code.msg(i) + "] (" + str + ")");
        return;
    }
    
    long getTicks() {
        return this.ticks;
    }
    
    @Override
    public long getTime() {
        return this.time;
    }
    
    void delay() {
        Thread.yield();
        try {
            Thread.sleep(200L);
        }
        catch (InterruptedException ex) {}
    }
    
    synchronized void startExchange() {
        ++this.exchangeCount;
    }
    
    synchronized int endExchange() {
        --this.exchangeCount;
        assert this.exchangeCount >= 0;
        return this.exchangeCount;
    }
    
    HttpServer getWrapper() {
        return this.wrapper;
    }
    
    void requestStarted(HttpConnection httpConnection) {
        httpConnection.creationTime = this.getTime();
        httpConnection.setState(HttpConnection.State.REQUEST);
        this.reqConnections.add(httpConnection);
    }
    
    void requestCompleted(HttpConnection httpConnection) {
        assert httpConnection.getState() == HttpConnection.State.REQUEST;
        this.reqConnections.remove(httpConnection);
        httpConnection.rspStartedTime = this.getTime();
        this.rspConnections.add(httpConnection);
        httpConnection.setState(HttpConnection.State.RESPONSE);
        return;
    }
    
    void responseCompleted(HttpConnection httpConnection) {
        assert httpConnection.getState() == HttpConnection.State.RESPONSE;
        this.rspConnections.remove(httpConnection);
        httpConnection.setState(HttpConnection.State.IDLE);
        return;
    }
    
    void logStackTrace(String msg) {
        this.logger.finest(msg);
        StringBuilder sb = new StringBuilder();
        StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
        for (int i = 0; i < stackTrace.length; ++i) {
            sb.append(stackTrace[i].toString()).append("\n");
        }
        this.logger.finest(sb.toString());
    }
    
    static long getTimeMillis(long n) {
        if (n == -1L) {
            return -1L;
        }
        return n * 1000L;
    }
    
    static void access$1100(ServerImpl serverImpl, HttpConnection httpConnection) {
        serverImpl.closeConnection(httpConnection);
    }
    
    static {
        CLOCK_TICK = ServerConfig.getClockTick();
        IDLE_INTERVAL = ServerConfig.getIdleInterval();
        MAX_IDLE_CONNECTIONS = ServerConfig.getMaxIdleConnections();
        TIMER_MILLIS = ServerConfig.getTimerMillis();
        MAX_REQ_TIME = getTimeMillis(ServerConfig.getMaxReqTime());
        MAX_RSP_TIME = getTimeMillis(ServerConfig.getMaxRspTime());
        timer1Enabled = (ServerImpl.MAX_REQ_TIME != -1L || ServerImpl.MAX_RSP_TIME != -1L);
        ServerImpl.debug = ServerConfig.debugEnabled();
    }
    
    private static class DefaultExecutor implements Executor
    {
        @Override
        public void execute(Runnable runnable) {
            runnable.run();
        }
    }
    
    class Dispatcher implements Runnable
    {
        final LinkedList<HttpConnection> connsToRegister;
        
        Dispatcher() {
            this.connsToRegister = new LinkedList();
        }
        
        private void handleEvent(Event event) {
            ExchangeImpl exchange = event.exchange;
            HttpConnection connection = exchange.getConnection();
            try {
                if ((event instanceof WriteFinishedEvent)) {
                    int endExchange = ServerImpl.this.endExchange();
                    if (ServerImpl.this.terminating && endExchange == 0) {
                        ServerImpl.this.finished = true;
                    }
                    ServerImpl.this.responseCompleted(connection);
                    LeftOverInputStream originalInputStream = exchange.getOriginalInputStream();
                    if (!originalInputStream.isEOF()) {
                        exchange.close = true;
                    }
                    if (exchange.close || ServerImpl.this.idleConnections.size() >= ServerImpl.MAX_IDLE_CONNECTIONS) {
                        connection.close();
                        ServerImpl.this.allConnections.remove(connection);
                    }
                    else if (originalInputStream.isDataBuffered()) {
                        ServerImpl.this.requestStarted(connection);
                        this.handle(connection.getChannel(), connection);
                    }
                    else {
                        this.connsToRegister.add(connection);
                    }
                }
            }
            catch (IOException thrown) {
                ServerImpl.this.logger.log(Level.FINER, "Dispatcher (1)", thrown);
                connection.close();
            }
        }
        
        void reRegister(HttpConnection ob) {
            try {
                SocketChannel channel = ob.getChannel();
                channel.configureBlocking(false);
                SelectionKey register = channel.register(ServerImpl.this.selector, 1);
                register.attach(ob);
                ob.selectionKey = register;
                ob.time = ServerImpl.this.getTime() + ServerImpl.IDLE_INTERVAL;
                ServerImpl.this.idleConnections.add(ob);
            }
            catch (IOException thrown) {
                ServerImpl.dprint(thrown);
                ServerImpl.this.logger.log(Level.FINER, "Dispatcher(8)", thrown);
                ob.close();
            }
        }
        
        @Override
        public void run() {
            while (!ServerImpl.this.finished) {
                try {
                    List<Object> access$900 = null;
                    synchronized (ServerImpl.this.lolock) {
                        if (ServerImpl.this.events.size() > 0) {
                            access$900 = ServerImpl.this.events;
                            ServerImpl.this.events = new LinkedList();
                        }
                    }
                    if (access$900 != null) {
                        Iterator<Object> iterator = access$900.iterator();
                        while (iterator.hasNext()) {
                            this.handleEvent((Event)iterator.next());
                        }
                    }
                    Iterator<Object> iterator2 = this.connsToRegister.iterator();
                    while (iterator2.hasNext()) {
                        this.reRegister((HttpConnection)iterator2.next());
                    }
                    this.connsToRegister.clear();
                    ServerImpl.this.selector.select(1000L);
                    Iterator<SelectionKey> iterator3 = ServerImpl.this.selector.selectedKeys().iterator();
                    while (iterator3.hasNext()) {
                        SelectionKey selectionKey = (SelectionKey)iterator3.next();
                        iterator3.remove();
                        if (selectionKey.equals(ServerImpl.this.listenerKey)) {
                            if (ServerImpl.this.terminating) {
                                continue;
                            }
                            SocketChannel accept = ServerImpl.this.schan.accept();
                            if (ServerConfig.noDelay()) {
                                accept.socket().setTcpNoDelay(true);
                            }
                            if (accept == null) {
                                continue;
                            }
                            accept.configureBlocking(false);
                            SelectionKey register = accept.register(ServerImpl.this.selector, 1);
                            HttpConnection ob = new HttpConnection();
                            ob.selectionKey = register;
                            ob.setChannel(accept);
                            register.attach(ob);
                            ServerImpl.this.requestStarted(ob);
                            ServerImpl.this.allConnections.add(ob);
                        }
                        else {
                            try {
                                if (selectionKey.isReadable()) {
                                    SocketChannel socketChannel = (SocketChannel)selectionKey.channel();
                                    HttpConnection httpConnection = (HttpConnection)selectionKey.attachment();
                                    selectionKey.cancel();
                                    socketChannel.configureBlocking(true);
                                    if (ServerImpl.this.idleConnections.remove(httpConnection)) {
                                        ServerImpl.this.requestStarted(httpConnection);
                                    }
                                    this.handle(socketChannel, httpConnection);
                                }
                                else {
                                    assert false;
                                    continue;
                                }
                            }
                            catch (CancelledKeyException ex) {
                                this.handleException(selectionKey, null);
                            }
                            catch (IOException ex2) {
                                this.handleException(selectionKey, ex2);
                            }
                        }
                    }
                    ServerImpl.this.selector.selectNow();
                }
                catch (IOException thrown) {
                    ServerImpl.this.logger.log(Level.FINER, "Dispatcher (4)", thrown);
                }
                catch (Exception thrown2) {
                    ServerImpl.this.logger.log(Level.FINER, "Dispatcher (7)", thrown2);
                }
            }
            try {
                ServerImpl.this.selector.close();
            }
            catch (Exception ex3) {}
        }
        
        private void handleException(SelectionKey selectionKey, Exception thrown) {
            HttpConnection httpConnection = (HttpConnection)selectionKey.attachment();
            if (thrown != null) {
                ServerImpl.this.logger.log(Level.FINER, "Dispatcher (2)", thrown);
            }
            ServerImpl.access$1100(ServerImpl.this, httpConnection);
        }
        
        public void handle(SocketChannel socketChannel, HttpConnection httpConnection) throws IOException {
            try {
                ServerImpl.this.executor.execute(new ServerImpl.Exchange(socketChannel, ServerImpl.this.protocol, httpConnection));
            }
            catch (HttpError thrown) {
                ServerImpl.this.logger.log(Level.FINER, "Dispatcher (4)", thrown);
                ServerImpl.access$1100(ServerImpl.this, httpConnection);
            }
            catch (IOException thrown2) {
                ServerImpl.this.logger.log(Level.FINER, "Dispatcher (5)", thrown2);
                ServerImpl.access$1100(ServerImpl.this, httpConnection);
            }
        }
    }
    
    class Exchange implements Runnable
    {
        SocketChannel chan;
        HttpConnection connection;
        HttpContextImpl context;
        InputStream rawin;
        OutputStream rawout;
        String protocol;
        ExchangeImpl tx;
        HttpContextImpl ctx;
        boolean rejected;
        
        Exchange(SocketChannel chan, String protocol, HttpConnection connection) throws IOException {
            this.rejected = false;
            this.chan = chan;
            this.connection = connection;
            this.protocol = protocol;
        }
        
        @Override
        public void run() {
            this.context = this.connection.getHttpContext();
            SSLEngine sslEngine = null;
            String requestLine = null;
            SSLStreams sslStreams = null;
            try {
                int n;
                if (this.context != null) {
                    this.rawin = this.connection.getInputStream();
                    this.rawout = this.connection.getRawOutputStream();
                    n = false;
                }
                else {
                    n = true;
                    if (ServerImpl.this.https) {
                        if (ServerImpl.this.sslContext == null) {
                            ServerImpl.this.logger.warning("SSL connection received. No https contxt created");
                            throw new HttpError("No SSL context established");
                        }
                        sslStreams = new SSLStreams(ServerImpl.this, ServerImpl.this.sslContext, this.chan);
                        this.rawin = sslStreams.getInputStream();
                        this.rawout = sslStreams.getOutputStream();
                        sslEngine = sslStreams.getSSLEngine();
                        this.connection.sslStreams = sslStreams;
                    }
                    else {
                        this.rawin = new BufferedInputStream(new Request.ReadStream(ServerImpl.this, this.chan));
                        this.rawout = new Request.WriteStream(ServerImpl.this, this.chan);
                    }
                    this.connection.raw = this.rawin;
                    this.connection.rawout = this.rawout;
                }
                Request o = new Request(this.rawin, this.rawout);
                requestLine = o.requestLine();
                if (requestLine == null) {
                    ServerImpl.access$1100(ServerImpl.this, this.connection);
                    return;
                }
                int index = requestLine.indexOf(32);
                if (index == -1) {
                    this.reject(400, requestLine, "Bad request line");
                    return;
                }
                String substring = requestLine.substring(0, index);
                int n2 = index + 1;
                int index2 = requestLine.indexOf(32, n2);
                if (index2 == -1) {
                    this.reject(400, requestLine, "Bad request line");
                    return;
                }
                URI o2 = new URI(requestLine.substring(n2, index2));
                String substring2 = requestLine.substring(index2 + 1);
                Headers headers = o.headers();
                String first = headers.getFirst("Transfer-encoding");
                long long1 = 0L;
                if (first != null && first.equalsIgnoreCase("chunked")) {
                    long1 = -1L;
                }
                else {
                    String first2 = headers.getFirst("Content-Length");
                    if (first2 != null) {
                        long1 = Long.parseLong(first2);
                    }
                    if (long1 == 0L) {
                        ServerImpl.this.requestCompleted(this.connection);
                    }
                }
                this.ctx = ServerImpl.this.contexts.findContext(this.protocol, o2.getPath());
                if (this.ctx == null) {
                    this.reject(404, requestLine, "No context found for request");
                    return;
                }
                this.connection.setContext(this.ctx);
                if (this.ctx.getHandler() == null) {
                    this.reject(500, requestLine, "No handler for context");
                    return;
                }
                this.tx = new ExchangeImpl(substring, o2, o, long1, this.connection);
                String first3 = headers.getFirst("Connection");
                Headers responseHeaders = this.tx.getResponseHeaders();
                if (first3 != null && first3.equalsIgnoreCase("close")) {
                    this.tx.close = true;
                }
                if (substring2.equalsIgnoreCase("http/1.0")) {
                    this.tx.http10 = true;
                    if (first3 == null) {
                        this.tx.close = true;
                        responseHeaders.set("Connection", "close");
                    }
                    else if (first3.equalsIgnoreCase("keep-alive")) {
                        responseHeaders.set("Connection", "keep-alive");
                        responseHeaders.set("Keep-Alive", "timeout=" + (int)(ServerConfig.getIdleInterval() / 1000L) + ", max=" + ServerConfig.getMaxIdleConnections());
                    }
                }
                if (!(n == 0)) {
                    this.connection.setParameters(this.rawin, this.rawout, this.chan, sslEngine, sslStreams, ServerImpl.this.sslContext, this.protocol, this.ctx, this.rawin);
                }
                String first4 = headers.getFirst("Expect");
                if (first4 != null && first4.equalsIgnoreCase("100-continue")) {
                    ServerImpl.this.logReply(100, requestLine, null);
                    this.sendReply(100, false, null);
                }
                Filter.Chain o3 = new Filter.Chain(this.ctx.getFilters(), new ServerImpl.Exchange.LinkHandler(new Filter.Chain(this.ctx.getSystemFilters(), this.ctx.getHandler())));
                this.tx.getRequestBody();
                this.tx.getResponseBody();
                if (ServerImpl.this.https) {
                    o3.doFilter(new HttpsExchangeImpl(this.tx));
                }
                else {
                    o3.doFilter(new HttpExchangeImpl(this.tx));
                }
            }
            catch (IOException thrown) {
                ServerImpl.this.logger.log(Level.FINER, "ServerImpl.Exchange (1)", thrown);
                ServerImpl.access$1100(ServerImpl.this, this.connection);
            }
            catch (NumberFormatException ex) {
                this.reject(400, requestLine, "NumberFormatException thrown");
            }
            catch (URISyntaxException ex2) {
                this.reject(400, requestLine, "URISyntaxException thrown");
            }
            catch (Exception thrown2) {
                ServerImpl.this.logger.log(Level.FINER, "ServerImpl.Exchange (2)", thrown2);
                ServerImpl.access$1100(ServerImpl.this, this.connection);
            }
        }
        
        void reject(int i, String s, String str) {
            this.rejected = true;
            ServerImpl.this.logReply(i, s, str);
            this.sendReply(i, false, "<h1>" + i + Code.msg(i) + "</h1>" + str);
            ServerImpl.access$1100(ServerImpl.this, this.connection);
        }
        
        void sendReply(int i, boolean b, String str) {
            try {
                StringBuilder sb = new StringBuilder(512);
                sb.append("HTTP/1.1 ").append(i).append(Code.msg(i)).append("\r\n");
                if (str != null && str.length() != 0) {
                    sb.append("Content-Length: ").append(str.length()).append("\r\n").append("Content-Type: text/html\r\n");
                }
                else {
                    sb.append("Content-Length: 0\r\n");
                    str = "";
                }
                if (b) {
                    sb.append("Connection: close\r\n");
                }
                sb.append("\r\n").append(str);
                this.rawout.write(sb.toString().getBytes("ISO8859_1"));
                this.rawout.flush();
                if (b) {
                    ServerImpl.access$1100(ServerImpl.this, this.connection);
                }
            }
            catch (IOException thrown) {
                ServerImpl.this.logger.log(Level.FINER, "ServerImpl.sendReply", thrown);
                ServerImpl.access$1100(ServerImpl.this, this.connection);
            }
        }
        
        class LinkHandler implements HttpHandler
        {
            Filter.Chain nextChain;
            
            LinkHandler(Filter.Chain nextChain) {
                this.nextChain = nextChain;
            }
            
            @Override
            public void handle(HttpExchange exchange) throws IOException {
                this.nextChain.doFilter(exchange);
            }
        }
    }
    
    class ServerTimerTask extends TimerTask
    {
        @Override
        public void run() {
            LinkedList<HttpConnection> list = new LinkedList();
            ServerImpl.this.time = System.currentTimeMillis();
            ServerImpl.this.ticks++;
            synchronized (ServerImpl.this.idleConnections) {
                for (HttpConnection e : ServerImpl.this.idleConnections) {
                    if (e.time <= ServerImpl.this.time) {
                        list.add(e);
                    }
                    else {
                        continue;
                    }
                }
                for (HttpConnection httpConnection : list) {
                    ServerImpl.this.idleConnections.remove(httpConnection);
                    ServerImpl.this.allConnections.remove(httpConnection);
                    httpConnection.close();
                }
            }
        }
    }
    
    class ServerTimerTask1 extends TimerTask
    {
        @Override
        public void run() {
            LinkedList<HttpConnection> list = new LinkedList();
            ServerImpl.this.time = System.currentTimeMillis();
            synchronized (ServerImpl.this.reqConnections) {
                if (ServerImpl.MAX_REQ_TIME != -1L) {
                    for (HttpConnection e : ServerImpl.this.reqConnections) {
                        if (e.creationTime + ServerImpl.TIMER_MILLIS + ServerImpl.MAX_REQ_TIME <= ServerImpl.this.time) {
                            list.add(e);
                        }
                        else {
                            continue;
                        }
                    }
                    for (HttpConnection obj : list) {
                        ServerImpl.this.logger.log(Level.FINE, "closing: no request: " + obj);
                        ServerImpl.this.reqConnections.remove(obj);
                        ServerImpl.this.allConnections.remove(obj);
                        obj.close();
                    }
                }
            }
            LinkedList<HttpConnection> list2 = new LinkedList();
            synchronized (ServerImpl.this.rspConnections) {
                if (ServerImpl.MAX_RSP_TIME != -1L) {
                    for (HttpConnection e2 : ServerImpl.this.rspConnections) {
                        if (e2.rspStartedTime + ServerImpl.TIMER_MILLIS + ServerImpl.MAX_RSP_TIME <= ServerImpl.this.time) {
                            list2.add(e2);
                        }
                        else {
                            continue;
                        }
                    }
                    for (HttpConnection obj2 : list2) {
                        ServerImpl.this.logger.log(Level.FINE, "closing: no response: " + obj2);
                        ServerImpl.this.rspConnections.remove(obj2);
                        ServerImpl.this.allConnections.remove(obj2);
                        obj2.close();
                    }
                }
            }
        }
    }
}
